@@ -4,6 +4,7 @@ Base class for finding modules.
 
 import dis
 import imp
+import logging
 import marshal
 import opcode
 import os
@@ -33,14 +34,6 @@ try:
 except NameError:
     bytes = str
 
-try:
-    isidentifier = str.isidentifier  # Built in method in Python 3
-except AttributeError:
-    # Check with regex for Python 2
-    _identifier_re = re.compile(r'^[a-z_]\w*$', re.I)
-    def isidentifier(s):
-        return bool(_identifier_re.match(s))
-
 try:
     source_from_cache = imp.source_from_cache  # Python 3.2 and above
 except AttributeError:
@@ -137,8 +130,7 @@ class ZipModulesCache(object):
 class ModuleFinder(object):
 
     def __init__(self, includeFiles = None, excludes = [], path = None,
-            replacePaths = None, copyDependentFiles = True, bootstrap = False,
-            compress = True):
+            replacePaths = None):
         self.includeFiles = includeFiles
         if includeFiles is None:
             self.includeFiles = []
@@ -146,8 +138,6 @@ class ModuleFinder(object):
         self.replacePaths = replacePaths
         if replacePaths is None:
             self.replacePaths = []
-        self.copyDependentFiles = copyDependentFiles
-        self.compress = compress
         self.path = path or sys.path
         self.modules = []
         self.aliases = {}
@@ -158,60 +148,30 @@ class ModuleFinder(object):
         cx_Freeze.hooks.initialize(self)
         initialExcludedModules = self.excludes.copy()
         self._AddBaseModules()
-        if not bootstrap:
-            self._ClearBaseModuleCode(initialExcludedModules)
 
     def _AddBaseModules(self):
         """Add the base modules to the finder. These are the modules that
            Python imports itself during initialization and, if not found,
            can result in behavior that differs from running from source;
            also include modules used within the bootstrap code.
-           
+
            When cx_Freeze is built, these modules (and modules they load) are
-           embedded into the base executables (see the WriteSourceFile method).
-           
-           When freezing applications, these modules are added, but their
-           Module objects are then cleared by _ClearBaseModules, so they are not
-           copied into a zip file. They will be accessible to the application as
-           frozen modules.
+           included in the startup zip file.
            """
-        self.ExcludeModule("cStringIO")
-        self.ExcludeModule("doctest")
-        self.ExcludeModule("getopt")
-        self.ExcludeModule("logging")
-        if sys.version_info[0] < 3:
-            # On Python 3, traceback -> linecache -> tokenize -> re, so
-            # we need to let it be loaded.
-            self.ExcludeModule("re")
-        self.ExcludeModule("subprocess")
         self.IncludeModule("traceback")
         self.IncludeModule("warnings")
         self.IncludePackage("encodings")
         if sys.version_info[0] >= 3:
             self.IncludeModule("io")
-        if sys.version_info[:2] >= (3, 3) and sys.version_info < (3, 5):
-            self.AddAlias("_frozen_importlib", "importlib._bootstrap")
-            self.IncludeModule("_frozen_importlib")
-            # importlib itself must not be frozen
-            del self._modules["importlib"]
-            del self._modules["importlib._bootstrap"]
-        if sys.version_info[:2] >= (3, 5):
-            self.AddAlias("_frozen_importlib", "importlib._bootstrap")
-            self.AddAlias("_frozen_importlib_external", "importlib._bootstrap_external")
-            self.IncludeModule("_frozen_importlib_external")
-            self.IncludeModule("_frozen_importlib")
-            # importlib itself must not be frozen
-            del self._modules["importlib"]
-            del self._modules["importlib._bootstrap"]
-            del self._modules["importlib._bootstrap_external"]
-        if self.copyDependentFiles:
-            self.IncludeModule("os")
-            self.IncludeModule("sys")
-            self.IncludeModule("zlib")
+        self.IncludeModule("os")
+        self.IncludeModule("sys")
+        self.IncludeModule("zlib")
         if sys.version_info[:2] >= (3, 4):
             # We need this, because collections gets loaded (via traceback),
             # and a partially frozen package causes problems.
             self.IncludeModule("collections.abc")
+        if sys.version_info[:2] >= (3,5):
+            self.IncludeModule("importlib.abc")
 
     def _AddModule(self, name):
         """Add a module to the list of modules but if one is already found,
@@ -222,25 +182,11 @@ class ModuleFinder(object):
             module = self._modules[name] = Module(name)
             self.modules.append(module)
             if name in self._badModules:
+                logging.debug("Removing module [%s] from list of bad modules",
+                        name)
                 del self._badModules[name]
         return module
 
-    def _ClearBaseModuleCode(self, initialExcludedModules):
-        """Clear the code for all of the base modules. This is done when not in
-           bootstrap mode so that the base modules are not included in the
-           zip file."""
-        for name in self.excludes:
-            if name in initialExcludedModules:
-                continue
-            del self._modules[name]
-        self.excludes = initialExcludedModules
-        for module in self._modules.values():
-            if module is None:
-                continue
-            if module.code is not None:
-                module.code = None
-                module.file = None
-
     def _DetermineParent(self, caller):
         """Determine the parent to use when searching packages."""
         if caller is not None:
@@ -295,12 +241,13 @@ class ModuleFinder(object):
     def _ImportAllSubModules(self, module, deferredImports, recursive = True):
         """Import all sub modules to the given package."""
         suffixes = [s[0] for s in imp.get_suffixes()]
+
         for path in module.path:
             try:
                 fileNames = os.listdir(path)
             except os.error:
                 continue
-            
+
             for fileName in fileNames:
                 fullName = os.path.join(path, fileName)
                 if os.path.isdir(fullName):
@@ -314,23 +261,30 @@ class ModuleFinder(object):
                     for suffix in suffixes:
                         if fileName.endswith(suffix):
                             name = fileName[:-len(suffix)]
-                            # Only modules with valid Python names are importable
-                            if isidentifier(name):
+
+                            # Skip modules whose names appear to contain '.',
+                            # as we may be using the wrong suffix, and even if
+                            # we're not, such module names will break the import
+                            # code.
+                            if "." not in name:
                                 break
+
                     else:
                         continue
                     if name == "__init__":
                         continue
-                    
+
                 subModuleName = "%s.%s" % (module.name, name)
                 subModule = self._InternalImportModule(subModuleName,
                                 deferredImports)
                 if subModule is None:
-                    raise ImportError("No module named %r" % subModuleName)
-                module.globalNames[name] = None
-                if subModule.path and recursive:
-                    self._ImportAllSubModules(subModule, deferredImports,
-                            recursive)
+                    if subModule not in self.excludes:
+                        raise ImportError("No module named %r" % subModuleName)
+                else:
+                    module.globalNames[name] = None
+                    if subModule.path and recursive:
+                        self._ImportAllSubModules(subModule, deferredImports,
+                                recursive)
 
     def _ImportDeferredImports(self, deferredImports, skipInImport = False):
         """Import any sub modules that were deferred, if applicable."""
@@ -342,6 +296,7 @@ class ModuleFinder(object):
                 self._EnsureFromList(caller, packageModule, subModuleNames,
                         newDeferredImports)
             deferredImports = newDeferredImports
+            skipInImport = True
 
     def _ImportModule(self, name, deferredImports, caller = None,
             relativeImportIndex = 0, namespace = False):
@@ -409,13 +364,14 @@ class ModuleFinder(object):
             return self._modules[name]
         except KeyError:
             pass
-        
+
         if name in self._builtinModules:
             module = self._AddModule(name)
+            logging.debug("Adding module [%s] [C_BUILTIN]", name)
             self._RunHook("load", module.name, module)
             module.inImport = False
             return module
-        
+
         pos = name.rfind(".")
         if pos < 0:  # Top-level module
             path = self.path
@@ -432,18 +388,21 @@ class ModuleFinder(object):
                 parentModule.ExtendPath()
             path = parentModule.path
             searchName = name[pos + 1:]
-        
+
         if name in self.aliases:
             actualName = self.aliases[name]
             module = self._InternalImportModule(actualName, deferredImports)
             self._modules[name] = module
             return module
-        
+
         try:
             fp, path, info = self._FindModule(searchName, path, namespace)
+            if info[-1] == imp.C_BUILTIN and parentModule is not None:
+                return None
             module = self._LoadModule(name, fp, path, info, deferredImports,
                     parentModule, namespace)
         except ImportError:
+            logging.debug("Module [%s] cannot be imported", name)
             self._modules[name] = None
             return None
         return module
@@ -458,8 +417,9 @@ class ModuleFinder(object):
         module = self._AddModule(name)
         module.file = path
         module.parent = parent
-        
+
         if type == imp.PY_SOURCE:
+            logging.debug("Adding module [%s] [PY_SOURCE]", name)
             # Load & compile Python source code
             if sys.version_info[0] >= 3:
                 # For Python 3, read the file with the correct encoding
@@ -476,6 +436,7 @@ class ModuleFinder(object):
                 raise ImportError("Invalid syntax in %s" % path)
         
         elif type == imp.PY_COMPILED:
+            logging.debug("Adding module [%s] [PY_COMPILED]", name)
             # Load Python bytecode
             if isinstance(fp, bytes):
                 magic = fp[:4]
@@ -491,11 +452,13 @@ class ModuleFinder(object):
                 fp.read(skip_bytes)
                 module.code = marshal.load(fp)
         
-        elif type == imp.C_EXTENSION and parent is not None:
-            # Our extension loader (see the freezer module) uses imp to load
-            # compiled extensions.
-            self.IncludeModule("imp")
-        
+        elif type == imp.C_EXTENSION:
+            logging.debug("Adding module [%s] [C_EXTENSION]", name)
+            if parent is None:
+                # Our extension loader (see the freezer module) uses imp to
+                # load compiled extensions.
+                self.IncludeModule("imp")
+
         # If there's a custom hook for this module, run it.
         self._RunHook("load", module.name, module)
         
@@ -520,11 +483,13 @@ class ModuleFinder(object):
         try:
             fp, path, info = self._FindModule("__init__", module.path, False)
             self._LoadModule(name, fp, path, info, deferredImports, parent)
+            logging.debug("Adding module [%s] [PKG_DIRECTORY]", name)
         except ImportError:
             if not namespace:
                 raise
             fileName = os.path.join(path, "__init__.py")
             module.code = compile("", fileName, "exec")
+            logging.debug("Adding module [%s] [PKG_NAMESPACE_DIRECTORY]", name)
         return module
 
     def _ReplacePathsInCode(self, topLevelModule, co):
@@ -677,8 +642,7 @@ class ModuleFinder(object):
 
     def IncludeFiles(self, sourcePath, targetPath):
         """Include the files in the given directory in the target build."""
-        if self.copyDependentFiles:
-            self.includeFiles.append((sourcePath, targetPath))
+        self.includeFiles.append((sourcePath, targetPath))
 
     def IncludeModule(self, name, namespace = False):
         """Include the named module in the frozen executable."""
@@ -713,38 +677,6 @@ class ModuleFinder(object):
                              "may not be needed on this platform.\n")
             sys.stdout.write("\n")
 
-    def WriteSourceFile(self, fileName):
-        dirName = os.path.dirname(fileName)
-        if not os.path.isdir(dirName):
-            os.makedirs(dirName)
-        outfp = open(fileName, "w")
-        names = list(self._modules.keys())
-        names.sort()
-        modulesWritten = []
-        for name in names:
-            module = self._modules[name]
-            if module is None or module.code is None:
-                continue
-            mangledName = "__".join(name.split("."))
-            sys.stdout.write("adding base module named %s\n" % name)
-            code = marshal.dumps(module.code)
-            size = len(code)
-            if module.path:
-                size = -size
-            modulesWritten.append((name, mangledName, size))
-            outfp.write("unsigned char M_%s[] = {" % mangledName)
-            for i in range(0, len(code), 16):
-                outfp.write("\n\t")
-                for op in code[i:i + 16]:
-                    if not isinstance(op, int):
-                        op = ord(op)
-                    outfp.write("%d," % op)
-            outfp.write("\n};\n\n");
-        outfp.write("static struct _frozen gFrozenModules[] = {\n")
-        for name, mangledName, size in modulesWritten:
-            outfp.write('    {"%s", M_%s, %d},\n' % (name, mangledName, size))
-        outfp.write("    {0, 0, 0}\n};\n")
-
 
 class Module(object):
 
